"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
function mongooseSubquery(schema, options = {}) {
    const decodeQuery = function () {
        return __awaiter(this, void 0, void 0, function* () {
            const mongooseQuery = this;
            const query = mongooseQuery.getQuery();
            if (options.beforeInit) {
                yield options.beforeInit(mongooseQuery);
            }
            const referenceFields = {};
            const paths = mongooseQuery.model.schema.paths;
            Object.keys(paths).forEach(fieldKey => {
                const field = paths[fieldKey];
                if ((field.constructor.name === "ObjectId" && field.options.ref)) {
                    referenceFields[fieldKey] = field.options.ref;
                }
                else if (field.constructor.name === "SchemaArray" && (field.$embeddedSchemaType.constructor.name === "ObjectId" && field.$embeddedSchemaType.options.ref)) {
                    referenceFields[fieldKey] = field.$embeddedSchemaType.options.ref;
                }
            });
            const decodeRecursive = function (obj, parentKey) {
                return __awaiter(this, void 0, void 0, function* () {
                    yield Promise.all(Object.keys(obj).map((key) => __awaiter(this, void 0, void 0, function* () {
                        const value = obj[key];
                        if (value && typeof value === "object" && value.$subquery) {
                            const modelName = referenceFields[parentKey || key];
                            if (modelName) {
                                if (options.initQuery) {
                                    yield options.initQuery(mongooseQuery, parentKey || key, obj, modelName);
                                }
                                const referenceModel = mongooseQuery.model.db.model(modelName);
                                let subquery = referenceModel.where(value.$subquery).select("_id");
                                if (options.beforeQuery) {
                                    yield options.beforeQuery(subquery, mongooseQuery);
                                }
                                let operator = value.$operator || "$in";
                                delete value.$subquery;
                                delete value.$operator;
                                let res;
                                if (options.resolve) {
                                    res = yield options.resolve(subquery, mongooseQuery);
                                }
                                else {
                                    res = yield subquery.find();
                                }
                                const resIds = res.map(doc => doc.id);
                                value[operator] = resIds;
                            }
                        }
                        else if (value && typeof value === "object") {
                            yield decodeRecursive(value, !/^\$/.test(key) && !Array.isArray(obj) ? key : parentKey);
                        }
                    })));
                });
            };
            mongooseQuery.decodeSubquery = () => __awaiter(this, void 0, void 0, function* () {
                yield decodeRecursive(query);
                mongooseQuery.setQuery(query);
            });
            if (!options.preventRunning) {
                yield mongooseQuery.decodeSubquery();
            }
        });
    };
    schema.pre('count', decodeQuery);
    schema.pre('deleteMany', decodeQuery);
    schema.pre('deleteOne', decodeQuery);
    schema.pre('find', decodeQuery);
    schema.pre('findOne', decodeQuery);
    schema.pre('findOneAndDelete', decodeQuery);
    schema.pre('findOneAndRemove', decodeQuery);
    schema.pre('findOneAndUpdate', decodeQuery);
    schema.pre('remove', decodeQuery);
    schema.pre('update', decodeQuery);
    schema.pre('updateOne', decodeQuery);
    schema.pre('updateMany', decodeQuery);
}
exports.default = mongooseSubquery;
//# sourceMappingURL=index.js.map